@@ -1,6 +1,8 @@
 from typing import (
-    # Dict,
-    Any, Callable, Tuple,
+    Dict,
+    Any,
+    Callable,
+    Tuple,
     Type,
     Union,
     List,
@@ -22,94 +24,89 @@ from .directive import (
     DirectiveCache
 )
 
-from .common import (
-    meta_property,
+from .common import rolling_calc
+
+from .meta import (
+    ColumnInfo,
+
+    init_stock_metas,
     copy_stock_metas,
-    ensure_return_type,
-    rolling_calc,
-
-    KEY_ALIAS_MAP,
-    KEY_COLUMNS_INFO_MAP,
-    KEY_DIRECTIVES_CACHE
+    copy_clean_stock_metas,
+    ensure_return_type
 )
 
 
-class ColumnInfo:
-    size: int
-    directive: Directive
-    period: int
-
-    def __init__(
-        self,
-        size: int,
-        directive: Directive,
-        period: int
-    ) -> None:
-        self.size = size
-        self.directive = directive
-        self.period = period
-
-    def update(self, size) -> 'ColumnInfo':
-        """Creates a new ColumnInfo and update the size
-        """
-
-        return ColumnInfo(
-            size,
-            self.directive,
-            self.period
-        )
-
-    def __repr__(self) -> str:
-        return f'<ColumnInfo {self.directive}, size:{self.size}, period:{self.period}>'  # noqa: E501
-
-
 class StockDataFrame(DataFrame):
     """The wrapper class for `pandas.DataFrame`
 
     Args definitions are the same as `pandas.DataFrame`
     """
 
-    _create_column: bool
+    _stock_create_column: bool = False
+    _stock_indexer_slice: Optional[slice] = None
+    _stock_indexer_axis: int = 0
 
-    # _stock_aliases_map: Dict[str]
-    # _stock_columns_info_map: Dict[ColumnInfo]
-    # _stock_directives_cache: DirectiveCache
+    # Directive cache can be shared between instances,
+    # so declare as static property
+    _stock_directives_cache = DirectiveCache()
 
-    _stock_aliases_map = meta_property(
-        KEY_ALIAS_MAP, lambda: {}
-    )
-
-    _stock_columns_info_map = meta_property(
-        KEY_COLUMNS_INFO_MAP, lambda: {}
-    )
-
-    _stock_directives_cache = meta_property(
-        KEY_DIRECTIVES_CACHE, lambda: DirectiveCache()
-    )
+    _stock_aliases_map: Dict[str, str]
+    _stock_columns_info_map: Dict[str, ColumnInfo]
 
     # Methods that used by pandas and sub classes
     # --------------------------------------------------------------------
 
     @property
     def _constructor(_) -> Type['StockDataFrame']:
-        """This method overrides `DataFrame._constructor`
+        """
+        This method overrides `DataFrame._constructor`
         which ensures the return type of several DataFrame methods
         """
 
         return StockDataFrame
 
     def __finalize__(self, other, *args, **kwargs) -> 'StockDataFrame':
-        """This method overrides `DataFrame.__finalize__`
+        """
+        Propagate metadata from other to self.
+
+        This method overrides `DataFrame.__finalize__`
         which ensures the meta info of StockDataFrame
         """
 
         super().__finalize__(other, *args, **kwargs)
 
         if isinstance(other, StockDataFrame):
-            copy_stock_metas(other, self)
+            copy_clean_stock_metas(
+                other,
+                self,
+                other._stock_indexer_slice,
+                other._stock_indexer_axis
+            )
 
         return self
 
+    def _slice(self, slice_obj: slice, axis: int = 0) -> 'StockDataFrame':
+        """
+        This method is called in several cases, self.iloc[slice] for example
+
+        We mark the slice and axis here to prevent extra calculations
+        """
+
+        self._stock_indexer_slice = slice_obj
+        self._stock_indexer_axis = axis
+
+        try:
+            result = super()._slice(slice_obj, axis)
+        except Exception as e:
+            raise e
+        finally:
+            self._stock_indexer_slice = None
+            self._stock_indexer_axis = 0
+
+        return result
+
+    # --------------------------------------------------------------------
+
     def __init__(
         self,
         data=None,
@@ -128,8 +125,8 @@ class StockDataFrame(DataFrame):
 
         if isinstance(data, StockDataFrame):
             copy_stock_metas(data, self)
-
-        self._create_column = False
+        else:
+            init_stock_metas(self)
 
         if date_column:
             self[date_column] = to_datetime(self[date_column])
@@ -162,37 +159,43 @@ class StockDataFrame(DataFrame):
     # Public Methods of stock-pandas
     # --------------------------------------------------------------------
 
-    def get_column(self, key: str) -> Series:
-        """Gets the column directly from dataframe by key.
+    def get_column(self, name: str) -> Series:
+        """
+        Gets the column directly from dataframe by key.
 
         This method applies column name aliases before getting the value.
+
+        Args:
+            name (str): The name of the column
+
+        Returns:
+            np.Series
         """
 
-        origin_key = key
+        origin_name = name
 
-        if key in self._stock_aliases_map:
+        if name in self._stock_aliases_map:
             # Map alias, if the key is an alias
-            key = self._stock_aliases_map[key]
+            name = self._stock_aliases_map[name]
 
         try:
-            return self._get_item_cache(key)
+            return self._get_item_cache(name)
         except KeyError:
-            raise KeyError(f'column "{origin_key}" not found')
+            raise KeyError(f'column "{origin_name}" not found')
 
     def exec(
         self,
         directive_str: str,
         create_column: Optional[bool] = None
     ) -> np.ndarray:
-        """Executes the given directive and
-        returns a numpy ndarray according to the directive.
+        """
+        Executes the given directive and returns a numpy ndarray according to the directive.
 
         This method is **NOT** Thread-safe.
 
         Args:
-            directive (str): directive
-            create_column (:obj:`bool`, optional): whether we should create a
-            column for the calculated series.
+            directive_str (str): directive
+            create_column (:obj:`bool`, optional): whether we should create a column for the calculated series.
 
         Returns:
             np.ndarray
@@ -204,22 +207,21 @@ class StockDataFrame(DataFrame):
         # We should call self.exec() without `create_column`
         # inside command formulas
         explicit_create_column = isinstance(create_column, bool)
-        original_create_column = self._create_column
+        original_create_column = self._stock_create_column
 
         if explicit_create_column:
-            # Vscode extension pyright could not handle this, so type ignore
-            self._create_column = create_column  # type: ignore
+            self._stock_create_column = create_column
         else:
             # cases
             # 1. called by users
             # 2. or called by command formulas
-            create_column = self._create_column
+            create_column = self._stock_create_column
 
         series = self._calc(directive_str)
 
         if explicit_create_column:
             # Set back to default value, since we complete calculatiing
-            self._create_column = original_create_column
+            self._stock_create_column = original_create_column
 
         return series
 
@@ -228,7 +230,8 @@ class StockDataFrame(DataFrame):
         as_name: str,
         src_name: str
     ) -> None:
-        """Defines column alias or directive alias
+        """
+        Defines column alias or directive alias
 
         Args:
             as_name (str): the alias name
@@ -250,13 +253,19 @@ class StockDataFrame(DataFrame):
         self,
         directive_str: str
     ) -> str:
-        """Stringify a `Directive` and get its full name
-        which is the actual column of the dataframe
+        """
+        Stringify a `Directive` and get its full name which is the actual column of the dataframe
 
         Usage::
-            stock.directive_stringify('boll')
 
+            stock.directive_stringify('boll')
             # It gets "boll:20,close"
+
+        Args:
+            directive_str (str): directive
+
+        Returns:
+            str
         """
 
         return str(self._parse_directive(directive_str))
@@ -275,10 +284,17 @@ class StockDataFrame(DataFrame):
             size (int): the size of the rolling window
             on (str | Directive): along which the function should be applied
             apply (Callable): the 1-D function to apply
-            forward (:obj:`bool`, optional): whether we should look backward
-                (default value) to get each rolling window or not
-            fill (:obj:`any`): the value used to fill where there are
-                not enough items to form a rolling window
+            forward (:obj:`bool`, optional): whether we should look backward (default value) to get each rolling window or not
+            fill (:obj:`any`): the value used to fill where there are not enough items to form a rolling window
+
+        Returns:
+            np.ndarray
+
+        Usage::
+
+            stock.rolling_calc(5, 'high', max)
+            # Gets the 5-period highest of high value, which is equivalent to
+            stock.exec('hhv:5').to_numpy()
         """
 
         array = self[on].to_numpy()
@@ -428,13 +444,18 @@ class StockDataFrame(DataFrame):
     def _calc(self, directive_str: str) -> np.ndarray:
         directive = self._parse_directive(directive_str)
 
-        _, series = self._get_or_calc_series(directive, self._create_column)
+        _, series = self._get_or_calc_series(
+            directive,
+            self._stock_create_column
+        )
 
         return series
 
 
 METHODS_TO_ENSURE_RETURN_TYPE = [
-    ('append', False),
+    # TODO:
+    # astype needs special treatment
+    # astype(dict_like) could not ensure return type
     ('astype', True)
 ]
 
